import sys
import paramiko
import json
import configparser
import os
import subprocess
import time
sys.path.append('../')
from common.runlogger import *
from os import path

def runTest(test_name,log_dir,num_boots,dir):
    esx_host = getenvdata(dir,'esx','hostname')[0]
    esx_user= getenvdata(dir,'esx','user')[0]
    esx_password = getenvdata(dir,'esx','password')[0]
    testresult = False
    resultfile = open(log_dir+'/.result', 'w')
    checkQuickBootCompatible(esx_host,esx_user,esx_password)
    testresult = installVIB(esx_host,esx_user,esx_password)
    testresult1 = False
    if testresult:
        testresult = executeTest(num_boots,esx_host,esx_user,esx_password,log_dir)
        if int(num_boots) == 1:
          testresult1 = execReboot(num_boots,esx_host,esx_user,esx_password)
        else:
          testresult1 = True
    time.sleep(300)
    if testresult and testresult1:
        logging.info("Test PASSED")
        resultfile.write("Passed")
    else:
        logging.info("Test FAILED")
        resultfile.write("Failed")

def execReboot(num_boots,esx_host,esx_user,esx_password):
    tries = 3
    logging.info("Rebooting " + num_boots + " times")
    ssh = sshConnection(esx_host,esx_user,esx_password)
    command = "reboot"
    stdin, stdout, stderr = ssh.exec_command(command)
    time.sleep(300)
    while(tries >= 0):
            try:
                ssh = sshConnection(esx_host,esx_user,esx_password)
                return True
            except:
                time.sleep(300)
                tries = tries - 1
    return False

def checkQuickBootCompatible(esx_host,esx_user,esx_password):
    ssh = sshConnection(esx_host,esx_user,esx_password)
    command = "/usr/lib/vmware/loadesx/bin/loadESXCheckCompat.py"
    stdin, stdout, stderr = ssh.exec_command(command)
    lst = []
    while True:
        nextline = stdout.readline()
        if nextline == '':
            break
        logging.info(nextline)
        lst.append(nextline)
    compatible_output = ''.join(lst)
    logging.info("compatible_output : "+compatible_output)
    if 'This system is compatible with Quick Boot' in compatible_output:
        logging.info("Host is compatible with Quick Boot")
    else:
        logging.error("Host is not compatible with Quick Boot, \n"+
                      "Please make sure Host is compatible with quick boot before starting the test")

def installVIB(esx_host,esx_user,esx_password):
    install_vib_result = False
    ssh = sshConnection(esx_host,esx_user,esx_password)
    #Evaluating the ESXi host build to install the right vib if required
    command = "vmware -v"
    stdin, stdout, stderr = ssh.exec_command(command)
    while True:
        nextline = stdout.readline()
        if nextline == '':
            break
        logging.info(nextline)
        st=nextline.split(' ')
    buildNo = st[3].split("-")[1].strip()
    logging.info("ESXi build being Certified is "+buildNo)
    build_version = st[2]
    logging.info("ESXi Version being Certified is "+st[2])
    #Evaluating the ESXi host build signature
    command = "cat /usr/share/certs/vmware.cert | grep AKjAWyE1cSg2QzqX6Wyj"
    stdin, stdout, stderr = ssh.exec_command(command)
    if 'AKjAWyE1cSg2QzqX6Wyj' in stdout.readline():
        host_signed = "official"
        logging.info("Host is signed with official key")
    else:
        host_signed = "test"
        logging.info("Host is signed with test key")
    # Is loadesx-platform-qual-test vib installed on the host?
    command = "esxcli software vib list | grep -w loadesx-platform-qual-test"
    stdin, stdout, stderr = ssh.exec_command(command)
    lst = []
    while True:
        nextline = stdout.readline()
        if nextline == '':
            break
        logging.info(nextline)
        lst.append(nextline)
    str1=''.join(lst)
    if 'loadesx-platform-qual-test' in str1:
        logging.info("The loadesx-platform-qual-test vib is already installed on the host "+esx_host)

       # Copy and update loadesx-platform-qual-test.vib
        logging.info("Start Copying the loadesx-platform-qual-test vib to "+esx_host+" \n")
        sftp = ssh.open_sftp()
        if host_signed == "official" and ( build_version == "7.0.1" or build_version == "7.0.2" or build_version == "7.0.3" ):
            sftp.put(sys.path[0]+"/Bin/loadesx-platform-qual-test.vib", "/loadesx-platform-qual-test.vib")
        elif host_signed == "official" and build_version == "7.0.0":
            sftp.put(sys.path[0]+"/Bin/70GA_vib/loadesx-platform-qual-test.vib", "/loadesx-platform-qual-test.vib")
        else:
            sftp.put(sys.path[0]+"/Bin/test_vib/loadesx-platform-qual-test.vib", "/loadesx-platform-qual-test.vib")
        logging.info("Copying of loadesx-platform-qual-test vib done..\n")
        command = "esxcli software vib update -v /loadesx-platform-qual-test.vib"
        stdin, stdout, stderr = ssh.exec_command(command)
        list2 = []
        while True:
            nextline = stdout.readline()
            if nextline == '':
                break
            logging.info(nextline)
            list2.append(nextline)
        str2=''.join(list2)
        if 'Message: Operation finished successfully' in str2:
            logging.info("loadesx-platform-qual-test.vib updated successfully")
            install_vib_result = True
        elif 'Message: Host is not changed' in str2:
            logging.info("Already latest loadesx-platform-qual-test.vib installed")
            install_vib_result = True
        elif 'Could not find a trusted signer: self signed certificate' in str2:
            sftp.put(sys.path[0]+"/Bin/test_vib/loadesx-platform-qual-test.vib", "/loadesx-platform-qual-test.vib")
            sftp.close()
            logging.info("This vib is from prod branch..Copying loadesx-platform-qual-test vib from main branch..\n")
            logging.info("Copying is done..")
            command = "esxcli software vib update -v /loadesx-platform-qual-test.vib"
            stdin, stdout, stderr = ssh.exec_command(command)
            list3 = []
            while True:
                nextline = stdout.readline()
                if nextline == '':
                    break
                logging.info(nextline)
                list3.append(nextline)
            str3=''.join(list3)
            if 'Message: Operation finished successfully' in str3:
                logging.info("loadesx-platform-qual-test.vib installed successfully")
                install_vib_result = True
            elif 'Message: Host is not changed' in str2:
                logging.info("Already latest loadesx-platform-qual-test.vib installed")
                install_vib_result = True
            else:
                logging.error("Failed to update loadesx-platform-qual-test.vib")
                install_vib_result = False
        else:
            logging.error("Failed to update loadesx-platform-qual-test.vib")
            install_vib_result = False
    else:
        # Copy and install loadesx-platform-qual-test.vib
        logging.info("Start Copying the loadesx-platform-qual-test vib to "+esx_host+" \n")
        sftp = ssh.open_sftp()
        if host_signed == "official" and ( build_version == "7.0.1" or build_version == "7.0.2" or build_version == "7.0.3" ):
            sftp.put(sys.path[0]+"/Bin/loadesx-platform-qual-test.vib", "/loadesx-platform-qual-test.vib")
        elif host_signed == "official" and build_version == "7.0.0":
            sftp.put(sys.path[0]+"/Bin/70GA_vib/loadesx-platform-qual-test.vib", "/loadesx-platform-qual-test.vib")
        else:
            sftp.put(sys.path[0]+"/Bin/test_vib/loadesx-platform-qual-test.vib", "/loadesx-platform-qual-test.vib")
        logging.info("Copying of loadesx-platform-qual-test vib done..\n")
        command = "esxcli software vib install -v /loadesx-platform-qual-test.vib"
        stdin, stdout, stderr = ssh.exec_command(command)
        list2 = []
        while True:
            nextline = stdout.readline()
            if nextline == '':
                break
            logging.info(nextline)
            list2.append(nextline)
        str2=''.join(list2)
        if 'Message: Operation finished successfully' in str2:
            logging.info("loadesx-platform-qual-test.vib installed successfully")
            install_vib_result = True
        elif 'Could not find a trusted signer: self signed certificate' in str2:
            sftp.put(sys.path[0]+"/Bin/test_vib/loadesx-platform-qual-test.vib", "/loadesx-platform-qual-test.vib")
            sftp.close()
            logging.info("This vib is from prod branch..Copying loadesx-platform-qual-test vib from main branch..\n")
            logging.info("Copying is done..")
            command = "esxcli software vib install -v /loadesx-platform-qual-test.vib"
            stdin, stdout, stderr = ssh.exec_command(command)
            list3 = []
            while True:
                nextline = stdout.readline()
                if nextline == '':
                    break
                logging.info(nextline)
                list3.append(nextline)
            str3=''.join(list3)
            if 'Message: Operation finished successfully' in str3:
                logging.info("loadesx-platform-qual-test.vib installed successfully")
                install_vib_result = True
            else:
                logging.error("Failed to install loadesx-platform-qual-test.vib")
                install_vib_result = False
        else:
            logging.error("Failed to install loadesx-platform-qual-test.vib")
            install_vib_result = False
        sftp.close()
    ssh.close()
    return install_vib_result


def executeTest(num_boots,esx_host,esx_user,esx_password,log_dir):
    ssh = sshConnection(esx_host,esx_user,esx_password)

    logging.info("Setting up Testbed to enable serial logging")
    command = 'esxcli system settings advanced set -o "/Misc/LogToSerial" -i 1;esxcli system settings advanced set -o "/Misc/DebugLogToSerial" -i 1;esxcli system settings advanced set -o "/Misc/LogPort" -s "COM1";esxcli system settings kernel set -s "com1_baud" -v 115200'
    stdin, stdout, stderr = ssh.exec_command(command)
    if stdout.readline() == '':
        logging.info("Serial logging is enabled on the Host")
    else:
        logging.error("Serial logging couldn't get enabled")

    logging.info("Setting up Testbed to boot "+num_boots+" quick boots")
    command = "loadESXPlatformQualTestSetup.py -n "+num_boots+" --ignore-platform"
    stdin, stdout, stderr = ssh.exec_command(command,get_pty=True)

    logging.info("Validating Testbed to boot "+num_boots+" quick boots")
    result = False
    list3 = []
    while True:
        nextline = stdout.readline()
        if nextline == '':
            break
        logging.info(nextline)
        list3.append(nextline)
    str3=''.join(list3)
    logging.info("Debug :"+str3)
    if 'INFO: Test setup completed. Test sequence starts with the next reboot' in str3:
        logging.info("Test setup completed. Test sequence starts with the next reboot")
        result = True
        #logging.info("Please reboot the host manually to start the quick boots.., \n"+
                    #"Wait for all the reboots complted, let host comes online and then Run Full_Quick_Boot_Verification Test")
    else:
        result = False
        logging.error("Failed during Test Setup for Quick boot Test")

    return result

def sshConnection(esx_host,esx_user,esx_password):
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(hostname=esx_host, username=esx_user, password=esx_password)
    return ssh

def getenvdata(dir, hosttype,key):
      with open(dir+'runlist.json') as json_data:
          data = json.load(json_data)
          return  [ li[key] for li in data['environment']['hosts'] if li['hosttype'] == hosttype]

def main():
    runlogger(sys.argv[2])
    runTest(sys.argv[1],sys.argv[2],sys.argv[3],'/')


if __name__== '__main__':
    main()
